#! /usr/bin/env python3

#

#  Program name:  TRYCSV.py. 

#

#  This Python console program is used to convert CSV files exported from

#  Excel to a format that can be used with TRYSYS. 

# 

# Assumptions:

# 	1) The CSV file starts with a line of header strings

# 	2) The values are delimited by a TAB or a COMMA

#

#  Python Programmer: Tony Goelz

#  Original version date: 12/1/2024



import os, csv, sys



def getNumberOfValuesPerRow(csvFileName,valueSeperator):



	numRows = 0

	with open(csvFileName) as csvFile:

		csvReader =  csv.reader(csvFile, delimiter=valueSeperator)

		fields = next(csvReader)

		numRows = len(fields)

	# End With



	return numRows



def getFieldDescriptions(numValues):



	fieldDesc = []

	done = False

	which = "first"

	fieldName = "none"

	valuesLeft = numValues



	while done == False:

		print("\n  There are " + str(valuesLeft) + " unused data values remaining.")

		fieldType = "init"

		while fieldType != "" and fieldType != "F" and fieldType != 'A':

			print("  Enter the type of the " + which + " data field.")

			fieldType = input("  (enter F for numeric, A for character, or press Return to stop): ").upper()

		# End While



		which = "next"

		if fieldType == "":

			done = True

		elif fieldType == "A":

			fieldName = "character"

		else:

			fieldName = "numeric"

		# End If



		if done == False:

			if fieldType == 'F':

				outputRepeatCount = -1

				while done == False and outputRepeatCount == -1 and fieldType == "F":

					print("\n  Enter the number of input values (repeat count) to use with this numeric data field.")

					rsp = input("  (enter number of values as integer or press Return for 1): ").lstrip()

					if rsp != "":

						try:

							outputRepeatCount = int(rsp)

						except:

							print("     Error: \"" + rsp + "\" cannot be converted to an integer.")

					else:

						outputRepeatCount = 1

					# End If

					

					if (outputRepeatCount > valuesLeft):

						print("     Error: " + str(outputRepeatCount) + " exceeds number of remaining data values (" + str(valuesLeft) + ")")

					# End If

				# End While

			else:

				outputRepeatCount = 1

			# End If

		# End If

	

		outputFieldWidth = -1

		while done == False and outputFieldWidth == -1:

			print("\n  Enter the output width for this " + fieldName + " field in the converted file.")

			rsp = input("  (enter width as integer or press Return for 1): ").lstrip()

			if rsp != "":

				try:

					outputFieldWidth = int(rsp)

				except:

					print("     Error: \"" + rsp + "\" cannot be converted to an integer.")

			else:

				outputFieldWidth = 1

			# End If

		# End While



		if done == False:

			fieldDesc.append( (outputRepeatCount, fieldType, outputFieldWidth) )

			valuesLeft = valuesLeft - outputRepeatCount

		# End If

	# End While



	if valuesLeft > 0:

		print("\n  " + str(valuesLeft) + " data values remain unused and will not be transferred to the converted file.")

	# End If

	return fieldDesc



def create_TRYSYS_from_CSV(csvFileName, dataFileName, valueSeparator, fieldDescriptions):

# Transform a CSV file into something that can be descriped by a FORTRAN

# FORMAT statment for TRYSYS.      



	formatString = "("	

	numberOfFields = len(fieldDescriptions)

	warningIssued = False

	numberOFVariables = 0

    

	dataFile = open(dataFileName,"w")

	with open(csvFileName) as csvFile:	

		csvReader =  csv.reader(csvFile, delimiter=valueSeparator)

		headers = next(csvReader)

		formatCreated = False

		rowCount = 1

		for row in csvReader:

			rowCount = rowCount + 1

			columnIndex = 0

			fieldIndex = 0

			done = False



			while columnIndex < len(row) and done == False:

				fieldItem = fieldDescriptions[fieldIndex]

				fieldIndex = fieldIndex + 1

				if fieldIndex == numberOfFields:

					done = True

				# End If



				repeatCount = fieldItem[0]

				fieldType = fieldItem[1]

				fieldWidth = fieldItem[2]



				if formatCreated == False:

					if (formatString != "("):

						formatString = formatString + ","

					# End If



					if (fieldType == "A"):

							formatString = formatString + fieldType + str(fieldWidth)

					else:

							numberOFVariables = numberOFVariables + repeatCount

							if repeatCount > 1:

								formatString = formatString + str(repeatCount)

							# End If

							formatString = formatString + fieldType + str(fieldWidth) + ".0"

					# End If

				# End If



				if warningIssued == False and len(row[columnIndex]) > fieldWidth:

					print("\n ## WARNING: value \"" + row[columnIndex] + "\" for item " + str(columnIndex+1) + " in line " + str(rowCount) + " exceeds field width " + str(fieldWidth))

					print("\n")

					warningIssued = True

				# End If



				while repeatCount > 0:

					data = row[columnIndex]

					columnIndex = columnIndex + 1

					repeatCount = repeatCount - 1

					dataFile.write(data.rjust(fieldWidth,' '))

				# End While

			# End While

		

			dataFile.write("\n")

			rowCount = rowCount + 1

			if formatCreated == False:

				formatString = formatString + ")"

				formatCreated = True

			# End If

		# End For

	# End With



	dataFile.close()

	print("  Use " + str(numberOFVariables) + " variables and this FORMAT in the command file DAP record:")

	print("  " + formatString)



# Main starts here

#

# Windows

if sys.platform == 'win32':

    TRYSYS_DIRECTORY = 'C:/trysys'

# macOS

elif sys.platform == 'darwin':

    TRYSYS_DIRECTORY  = os.environ['HOME'] + '/Desktop/trysys'

# Linux

elif sys.platform == 'linux':

    TRYSYS_DIRECTORY = os.environ['HOME'] + '/trysys'

else:

    print('TRYTAKE: unsupported operating system ' + sys.platform)

# End If



# Normalize everything with OS specific path separators

TRYSYS_DIRECTORY = os.path.normpath(TRYSYS_DIRECTORY)



# Correct our working directory if needed. When double clicked from a GUI,

# we want to set our working directory to the location of TRYRUN.py but

# we do not want to do this when run from the command line, since the common

# command line usage is to add the TRYSYS programs folder to PATH and then

# run from the directory that contains the data files. 



cwd = os.getcwd() 

scriptdir = os.path.dirname(os.path.abspath(sys.argv[0]))

if cwd != scriptdir and scriptdir != TRYSYS_DIRECTORY:

    os.chdir(scriptdir)

# End If

      

# Prompt for the name of the CSV file to convert and ensure it exists

# and can be opened. 



print("\n  You are using the T R Y C S V console app to convert a CSV data file")

print("  to a format that can be read by TRYSYS. The input file is assumed to")

print("  begin with a single line of headers and to use either a TAB or a comma")

print("  as its value separator.")

finished = False

while not finished:

	csvFileName = input("\n  Enter the name of the CSV file to convert: ")

	finished = os.path.isfile(csvFileName)

	if not finished:

		print("\n   " + csvFileName + " not found.  Try again.") 

		print("      (Or close the console window and restart TRYCSV.) \n\n")

	else:

		try:

			testFile = open(csvFileName,"r")

			testFile.close()

			finished = True

		except:

			print("\n   Unable to open " +  csvFileName + " for reading. Try again. ") 

			print("      (Or close the console window and restart TRYCSV.) \n\n")

	# End If



# Prompt for the name to use for the converted output file and ensure it can be opened.

finished = False

while not finished:

	dataFileName = input("\n  Enter a name for the converted TRYSYS data file: ")

	try:

		testFile = open(dataFileName,"w")

		testFile.close()

		finished = True

	except:

		print("\n   Unable to create " +  dataFileName + " in current directory.  Try again. ") 

		print("      (Or close the console window and restart TRYCSV.) \n\n")

# End If



# Prompt for the value delimitor used in the CSV file.     

rsp = "init"

separator = "\t"

while rsp != "1" and rsp != "":

	print("\n  What value separator is used in " + csvFileName + "?")

	rsp = input("  (Enter 1 for comma or press Return for TAB): ")

# End While



if rsp == "1":

	separator = ","

# End If



numValues = getNumberOfValuesPerRow(csvFileName,separator)

print("\n  File " + csvFileName + " has " + str(numValues) + " values on each line.")

fieldDescriptions = []

fieldDescriptions = getFieldDescriptions(numValues)

if len(fieldDescriptions) == 0:

	print("  No fields entered, skipping conversion.")

else:

	create_TRYSYS_from_CSV(csvFileName, dataFileName, separator, fieldDescriptions) 

# End IF    



endit = input("\n  Press Return to exit the program.")

